@@ -4,7 +4,7 @@ package Lemonldap::NG::Portal::2F::Register::Base;
 use strict;
 use Mouse;
 
-our $VERSION = '2.0.16';
+our $VERSION = '2.18.0';
 
 extends 'Lemonldap::NG::Portal::Main::Plugin';
 
@@ -72,9 +72,12 @@ sub checkNameSfa {
     return $name;
 }
 
-sub checkCsrf {
-    my ( $self, $req ) = @_;
-
-    return $req->headers->header('X-CSRF-Check');
+# Update request with newly registered device
+sub markRegistered {
+    my ($self, $req)  = @_;
+    $req->data->{_2fRegistered}                     = 1;
+    $req->userData->{_2f}                           = $self->prefix;
+    $req->userData->{registeredAuthenticationLevel} = $self->authnLevel;
 }
+
 1;
